#include "core.h"
#include "simplestring.h"
#include "hashedstring.h"
#include "luainterpreter.h"
#include "lualibraries.h"
#include "packstream.h"

static int PkgAdd( lua_State *L )
{
	const char *PkgFile = luaL_checkstring( L, 1 );
	PackStream::StaticAddPackageFile( PkgFile );
	lua_pushboolean( L, PackStream::StaticPackageIsLoaded( PkgFile ) );
	return 1;
}

static int PkgList( lua_State *L )
{
	const char *PkgFile = luaL_checkstring( L, 1 );
	Array<SPackageFileEntry*>::Iterator Iter = PackStream::StaticListPackageEntries( PkgFile );
	if( Iter.IsValid() )
	{
		int cnt = 1;
		lua_newtable( L );
		while( Iter.IsValid() )
		{
			lua_pushstring( L, Iter.GetValue()->m_Filename.CStr() );
			lua_rawseti( L, -2, cnt++ );
			Iter++;
		}
	}
	else
	{
		lua_pushnil( L );
	}
	return 1;
}

static int PkgFileExists( lua_State *L )
{
	const char *File = luaL_checkstring( L, 1 );
	bool PkgOnly = false;
	if( lua_gettop( L ) > 1 )
	{
		PkgOnly = lua_toboolean( L, 2 );
	}
	lua_pushboolean( L, PackStream::StaticFileExists( File, PkgOnly ) );
	return 1;
}

/* Function list
 */
static const luaL_Reg Functions[] = {
	{ "add", PkgAdd },
	{ "list", PkgList },
	{ "fileexists", PkgFileExists },
 	{ NULL, NULL }
};

int LuaLibraries::PkgInit()
{
	LuaInterpreter *Lua = LuaInterpreter::GetInstance();
	return Lua->RegisterNamespace( "pkg", Functions );
}
