#include "core.h"
#include "array.h"
#include "wbevent.h"
#include "wbentity.h"
#include "wbactionstack.h"

static Array<const WBEvent*>* sStack;
static Array<const WBEntity*>* sReactingEntityStack;

void WBActionStack::Initialize()
{
	ASSERT( !sStack );
	sStack = new Array<const WBEvent*>;
	sStack->SetDeflate( false );
	sReactingEntityStack = new Array<const WBEntity*>;
	sReactingEntityStack->SetDeflate( false );
}

void WBActionStack::ShutDown()
{
	SafeDelete( sStack );
}

void WBActionStack::Push( const WBEvent& Event )
{
	ASSERT( sStack );
	sStack->PushBack( &Event );
}

void WBActionStack::Pop()
{
	ASSERT( sStack );
	sStack->PopBack();
}

const WBEvent& WBActionStack::Top()
{
	ASSERT( sStack );
	ASSERT( sStack->Size() );
	return *(sStack->Last());
}

void WBActionStack::PushReactionsSelf( WBEntity* const pReactingEntity )
{
	ASSERT( sReactingEntityStack );
	sReactingEntityStack->PushBack( pReactingEntity );
}

void WBActionStack::PopReactionsSelf()
{
	ASSERT( sReactingEntityStack );
	sReactingEntityStack->PopBack();
}

const WBEntity* WBActionStack::TopReactionsSelf()
{
	ASSERT( sReactingEntityStack );
	ASSERT( sReactingEntityStack->Size() );
	return sReactingEntityStack->Last();
}
