#include "core.h"
#include "wbactioneldpostscore.h"
#include "configmanager.h"
#include "wbactionstack.h"
#include "Components/wbcompeldsouls.h"
#include "ui.h"
#include "Common/uimanagercommon.h"
#include "Screens/uiscreen-inputdialog.h"
#include "eldritchframework.h"
#include "system.h"
#include "filestream.h"


WBActionEldPostScore::WBActionEldPostScore()
:	m_ScorePE()
,	m_LeaderboardTag()
{
}

WBActionEldPostScore::~WBActionEldPostScore()
{
}

/*virtual*/ void WBActionEldPostScore::InitializeFromDefinition( const SimpleString& DefinitionName )
{
	WBAction::InitializeFromDefinition( DefinitionName );

	MAKEHASH( DefinitionName );

	STATICHASH( ScorePE );
	const SimpleString ScorePEDef = ConfigManager::GetString( sScorePE, "", sDefinitionName );
	m_ScorePE.InitializeFromDefinition( ScorePEDef );

	STATICHASH( LeaderboardTag );
	m_LeaderboardTag = ConfigManager::GetString( sLeaderboardTag, "", sDefinitionName );
}

static void OKCallback( void *pDialog, void *pUserdata )
{
	UIScreenInputDialog *pInputDialog = reinterpret_cast<UIScreenInputDialog*>( pDialog );
	WBActionEldPostScore *pActionPostScore = reinterpret_cast<WBActionEldPostScore*>( pUserdata );
	pActionPostScore->PostScore( pInputDialog->GetString() );
}

/*virtual*/ void WBActionEldPostScore::Execute()
{
	WBAction::Execute();

	STATIC_HASHED_STRING( EventOwner );
	WBEntity* const pEntity = WBActionStack::Top().GetEntity( sEventOwner );
	ASSERT( pEntity );

	WBParamEvaluator::SPEContext PEContext;
	PEContext.m_Entity = pEntity;
	m_ScorePE.Evaluate( PEContext );

	SimpleString Username = System::GetUsername();
	PRINTF("Username: %s\n", Username.CStr());
	EldritchFramework::GetInstance()->GetUIManager()->ShowInputDialog( true, "Enter name for highscore table", "", Username, SUICallback( OKCallback, this ), SUICallback() );
}

struct SHiscoreEntry {
	SimpleString m_Name;
	int m_Score;

	SHiscoreEntry( SimpleString Name, int Score )
	: m_Name( Name )
	, m_Score( Score )
	{}

	bool operator<( const SHiscoreEntry &Other ) const
	{
		// we want to sort so that the highest score comes first!
		return m_Score > Other.m_Score;
	}
};

// TODO properly write to file
void WBActionEldPostScore::PostScore( SimpleString Username )
{
	if( Username == "" )
	{
		return;
	}
	PRINTF( "PostScore: %s: %s: %d\n", m_LeaderboardTag.CStr(), Username.CStr(), m_ScorePE.GetInt() );

	Array<SHiscoreEntry> HiscoreTable;
	SimpleString NamePattern = m_LeaderboardTag + "%dName";
	SimpleString ScorePattern = m_LeaderboardTag + "%dScore";

	VarMap ScoreMap;
	LoadScore( ScoreMap );

	uint i = 0;
	bool done = false;
	while( !done )
	{
		const char *Name = ScoreMap.GetString( SimpleString::PrintF( NamePattern, i ), NULL );
		if( !Name )
		{
			done = true;
			continue;
		}
		int Score = ScoreMap.GetInt( SimpleString::PrintF( ScorePattern, i ), 0 );
		HiscoreTable.PushBack( SHiscoreEntry( Name, Score ) );
		++i;
	}
	HiscoreTable.PushBack( SHiscoreEntry( Username, m_ScorePE.GetInt() ) );
	HiscoreTable.InsertionSort();

	for( i = 0; i < HiscoreTable.Size(); ++i )
	{
		PRINTF( "%s%i: %s: %d\n", m_LeaderboardTag.CStr(), i, HiscoreTable[i].m_Name.CStr(), HiscoreTable[i].m_Score );
		ScoreMap.SetString( SimpleString::PrintF( NamePattern, i ), HiscoreTable[i].m_Name.CStr() );
		ScoreMap.SetInt( SimpleString::PrintF( ScorePattern, i ), HiscoreTable[i].m_Score );
	}

	SaveScore( ScoreMap );
}

/* static */ SimpleString WBActionEldPostScore::GetScoreFile()
{
	const SimpleString SaveLoadPath	= EldritchFramework::GetInstance()->GetSaveLoadPath();
	return SaveLoadPath + "scores.meldritchvarmap";
}

/* static */ void WBActionEldPostScore::LoadScore( VarMap& Score )
{
	Score.Clear();
	SimpleString ScoreFile = GetScoreFile();
	FileStream Stream( ScoreFile.CStr(), FileStream::EFM_Read );
	if( Stream.EOS() )
	{
		return;
	}
	Score.Load( Stream );
}

/* static */ void WBActionEldPostScore::SaveScore( VarMap& Score )
{
	SimpleString ScoreFile = GetScoreFile();
	FileStream Stream( ScoreFile.CStr(), FileStream::EFM_Write );
	Score.Save( Stream );
}
