#include "core.h"
#include "simplestring.h"
#include "file.h"
#include "configmanager.h"
#include "printmanager.h"
#include "reversehash.h"
#include "cmdlineparser.h"
#include "luainterpreter.h"
#include "lualibraries.h"

#include <stdio.h>

void InitEldritchStuff()
{
	ReverseHash::Initialize();
	ConfigManager::GetInstance();
}

int ConfigReversehash( lua_State *L )
{
	uint32 Hash = luaL_checkinteger( L, 1 );
	SimpleString String = ReverseHash::ReversedHash( Hash );
	lua_pushstring( L, String.CStr() );
	return 1;
}

int ConfigRegisterhash( lua_State *L )
{
	SimpleString String = luaL_checkstring( L, 1 );
	HashedString Hash( String );
	ReverseHash::RegisterHash( Hash, String );
	return 0;
}

void RunCLI( LuaInterpreter *Lua )
{
	char Buffer[ BUFSIZ ];
	bool Done = false;
	while( !Done )
	{
		fputs( "mEldritchLua> ", stdout );
		if( fgets( Buffer, BUFSIZ, stdin ) != NULL )
		{
			if ( *Buffer == '=' )
			{
				memmove( Buffer + 7, Buffer + 1, BUFSIZ - 7);
				strncpy( Buffer, "return ", 7 );
			}

			Lua->Eval( Buffer, "input", true );
			if ( Lua->LastError() )
			{
				fprintf(stdout, "Error: %s\n", Lua->LastError() );
			}
			else
			{
				Lua->PrintStack();
			}
		}
		else
		{
			Done = true;
		}
	}
}

static int Printf( lua_State *L )
{
	if ( lua_gettop( L ) > 1 )
	{
		int nargs = lua_gettop( L );
		lua_getglobal( L, "string" );
		lua_getfield( L, -1, "format" );
		lua_insert( L, 1 );
		lua_pop( L, 1 );
		lua_call( L, nargs, 1 );
	}

	const char *Msg = luaL_checkstring(L, 1);
	puts( Msg );
	return 0;
}

CmdlineParser ParseCommandline( int Argc, char *Argv[] )
{
	CmdlineParser Opts( Argc, Argv );
	Opts.AddOption( 'i', "interactive", "run interactively, even after loading lua files" );
	Opts.AddPositionals( 0, -1, "file.lua [args...]", "lua files" );
	if ( !Opts.Parse() )
	{
		puts( Opts.Usage().CStr() );
		exit( 1 );
	}
	return Opts;
}

int main( int Argc, char **Argv )
{
	//SETPRINTLEVEL( PRINTLEVEL_Spam );
	LuaLibraries::Init();
	InitEldritchStuff();

	LuaInterpreter *Lua = LuaInterpreter::GetInstance();

	SimpleString Path = SimpleString("?.lua;Lua/?.lua;" ) + FileUtil::GetExecutableDirectory() + "/?.lua;" + FileUtil::GetExecutableDirectory() + "/Lua/?.lua;";
	Lua->SetString( "path", Path.CStr(), "package" );
	
	Lua->AddCFunction( "reversehash", ConfigReversehash, "config" );
	Lua->AddCFunction( "registerhash", ConfigRegisterhash, "config" );
	Lua->AddCFunction( "printf", Printf );

	CmdlineParser Opts = ParseCommandline( Argc, Argv );

	if( Opts.NumPositionals() > 0 )
	{
		lua_State *L = Lua->GetState();
		lua_newtable( L );
		for( int i = 1; i <= Opts.NumPositionals(); ++i )
		{
			lua_pushstring( L, Opts[ i ] );
			lua_rawseti( L, -2, i - 1 );
		}
		lua_setglobal( L, "ARG" );

		if ( !Lua->RunFile( Opts[ 1 ] ) )
		{
			printf( "Error: %s\n", Lua->LastError() );
			return 1;
		}
	}

	if ( Opts.NumPositionals() == 0 || Opts["interactive"] )
	{
		RunCLI( Lua );
	}

	return 0;
}
