#include "core.h"
#include "meshcompiler.h"
#include "map.h"
#include "array.h"
#include "cmdlineparser.h"
#include "file.h"
#include <string.h>
#include <stdio.h>

// This tool takes an XML file exported from Blender and does
// all the necessary wrangling to get it ready to load right
// into a game project.

// Syntax: MeshCompiler.exe inmesh.xml outmesh.cms -l	(-l specifies long indices; else shorts are used)

// Output format: (.cms for compiled mesh)
// Header: 22 bytes
//		4 bytes:	"DCMS" magic ID
//		4 bytes:	n, Number of vertices (unsigned long)
//		4 bytes:	i, Number of indices (unsigned long)
//		4 bytes:	f, Number of frames (unsigned long)
//		1 byte:		b, Number of bones (unsigned char)
//		1 byte:		a, Number of animations (unsigned char)
//		1 byte:		Long indices (bool)
//		1 byte:		Has UVs (bool)
//		1 byte:		Has normals (bool)
//		1 byte:		Has tangents (bool)
//		1 byte:		Has skeleton (bool)
// Array of n positions (Vector)
// Array of n UVs (Vector2) (optional)
// Array of n normals (Vector) (optional)
// Array of n tangents (Vector4) (optional)
// Array of n weights (VertexWeight) (optional)
// Array of i indices (unsigned shorts or unsigned long)
// Array of b bones per f frames (optional)
//		Bones consist of interlaced Quats and Vectors
//		This is in "frame-major order", or all bones for a frame packed together
// Array of a animations
//		Animations consist of a 32-byte name, an unsigned short start frame, and an unsigned short length

CmdlineParser ParseCommandline( int argc, char *argv[] )
{
	SimpleString ToolsDefault = FileUtil::GetExecutableDirectory() + "/tools.cfg";
	CmdlineParser Opts( argc, argv );
	Opts.AddOption( 'c', "toolscfg", "tools.cfg file", true, ToolsDefault.CStr() );
	Opts.AddOption( 'l', "longindices", "use long indices" );
	Opts.AddPositionals( 2, "<infile> <outfile>", "input and output files" );
	if ( !Opts.Parse() )
	{
		puts( Opts.Usage().CStr() );
		exit( 1 );
	}
	return Opts;
}

int main( int argc, char* argv[] )
{
	CmdlineParser Opts = ParseCommandline( argc, argv );

	bool LongIndices = false;
	if( Opts["longindices"] )
	{
		LongIndices = true;
	}

	MeshCompiler Compiler;
	return Compiler.Compile( Opts[1], Opts[2], LongIndices, Opts["toolscfg"] );
}